package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.data.IntegerSerializer;
import de.uni_marburg.powersort.sort.SortEnum;

import java.nio.file.Files;
import java.util.List;

public class Filter {
    /* Utility Class */
    private Filter() {
    }

    public static boolean isFiltered(DataEnum d, SortEnum s) {
        // To skip some of the inputs for all sort algorithms, uncomment them here.
//        if (List.of(
//                DataEnum.RANDOM_INTEGERS_300M,
//                DataEnum.ASCENDING_INTEGERS_300M,
//                DataEnum.DESCENDING_INTEGERS_250M,
//                DataEnum.ASCENDING_RUNS_1M,
//                DataEnum.ASCENDING_RUNS_WITH_OVERLAP_1M
//        ).contains(d)) {
//            return true;
//        }

        // To skip all competition inputs, uncomment the next line.
//        if(d.isCompetitionInput()){
//            return true;
//        }

        // Skip some sort algorithms for all competition inputs.
        if(d.isCompetitionInput()){
            if(List.of(
                    SortEnum.BUBBLE_SORT,
                    SortEnum.MERGE_SORT
            ).contains(s)){
                return true;
            }

            if(!Files.exists(IntegerSerializer.DATA_DIR)){
                System.err.println("Folder with inputs of competition missing: " + IntegerSerializer.DATA_DIR.toAbsolutePath());
                return true;
            }
        }

        //
        // Filter based on SortEnum values.
        //

        if (s == SortEnum.QUICK_SORT) {
            return List.of(
                    DataEnum.RANDOM_INTEGERS_300M,
                    DataEnum.ASCENDING_INTEGERS_300M,
                    DataEnum.DESCENDING_INTEGERS_250M,
                    DataEnum.ASCENDING_RUNS_1M,
                    DataEnum.ASCENDING_RUNS_WITH_OVERLAP_1M,

                    DataEnum.COMPETITION_34,
                    DataEnum.COMPETITION_35,
                    DataEnum.COMPETITION_36,
                    DataEnum.COMPETITION_37,
                    DataEnum.COMPETITION_38,
                    DataEnum.COMPETITION_39,
                    DataEnum.COMPETITION_83,
                    DataEnum.COMPETITION_85,
                    DataEnum.COMPETITION_153,
                    DataEnum.COMPETITION_163,
                    DataEnum.COMPETITION_164,
                    DataEnum.COMPETITION_165,
                    DataEnum.COMPETITION_166,
                    DataEnum.COMPETITION_167,
                    DataEnum.COMPETITION_168,
                    DataEnum.COMPETITION_169,
                    DataEnum.COMPETITION_170,
                    DataEnum.COMPETITION_171,
                    DataEnum.COMPETITION_172,
                    DataEnum.COMPETITION_173,
                    DataEnum.COMPETITION_174,
                    DataEnum.COMPETITION_181,
                    DataEnum.COMPETITION_182,
                    DataEnum.COMPETITION_183,
                    DataEnum.COMPETITION_184,
                    DataEnum.COMPETITION_185,
                    DataEnum.COMPETITION_186,
                    DataEnum.COMPETITION_187,
                    DataEnum.COMPETITION_205,
                    DataEnum.COMPETITION_206,
                    DataEnum.COMPETITION_207,
                    DataEnum.COMPETITION_213,
                    DataEnum.COMPETITION_214,
                    DataEnum.COMPETITION_216,
                    DataEnum.COMPETITION_218,
                    DataEnum.COMPETITION_220,
                    DataEnum.COMPETITION_222,
                    DataEnum.COMPETITION_225,
                    DataEnum.COMPETITION_226,
                    DataEnum.COMPETITION_228,
                    DataEnum.COMPETITION_231,
                    DataEnum.COMPETITION_232,
                    DataEnum.COMPETITION_235,
                    DataEnum.COMPETITION_236
            ).contains(d);
        }

        if (s == SortEnum.MERGE_SORT) {
            return List.of(
                    DataEnum.RANDOM_INTEGERS_300M,
                    DataEnum.ASCENDING_INTEGERS_300M,
                    DataEnum.DESCENDING_INTEGERS_250M
            ).contains(d);
        }

        if (s == SortEnum.BUBBLE_SORT) {
            return List.of(
                    DataEnum.DESCENDING_INTEGERS_250M,
                    DataEnum.ASCENDING_RUNS_1M,
                    DataEnum.ASCENDING_RUNS_WITH_OVERLAP_1M
            ).contains(d);
        }

        // TODO: Remove this once performance of FinnSort improved
        if (s == SortEnum.FINN_SORT) {
            return List.of(
                    DataEnum.DESCENDING_INTEGERS_250M,
                    DataEnum.ASCENDING_RUNS_1M,

                    DataEnum.COMPETITION_83,
                    DataEnum.COMPETITION_85,
                    DataEnum.COMPETITION_153,
                    DataEnum.COMPETITION_163,
                    DataEnum.COMPETITION_164,
                    DataEnum.COMPETITION_165,
                    DataEnum.COMPETITION_166,
                    DataEnum.COMPETITION_167,
                    DataEnum.COMPETITION_168,
                    DataEnum.COMPETITION_169,
                    DataEnum.COMPETITION_170,
                    DataEnum.COMPETITION_171,
                    DataEnum.COMPETITION_172,
                    DataEnum.COMPETITION_173,
                    DataEnum.COMPETITION_174,
                    DataEnum.COMPETITION_181,
                    DataEnum.COMPETITION_182,
                    DataEnum.COMPETITION_183,
                    DataEnum.COMPETITION_184,
                    DataEnum.COMPETITION_185,
                    DataEnum.COMPETITION_186,
                    DataEnum.COMPETITION_187,
                    DataEnum.COMPETITION_198,
                    DataEnum.COMPETITION_199,
                    DataEnum.COMPETITION_205,
                    DataEnum.COMPETITION_206,
                    DataEnum.COMPETITION_207,
                    DataEnum.COMPETITION_213,
                    DataEnum.COMPETITION_214,
                    DataEnum.COMPETITION_216,
                    DataEnum.COMPETITION_218,
                    DataEnum.COMPETITION_220,
                    DataEnum.COMPETITION_222,
                    DataEnum.COMPETITION_226,
                    DataEnum.COMPETITION_228,
                    DataEnum.COMPETITION_231,
                    DataEnum.COMPETITION_232,
                    DataEnum.COMPETITION_236
            ).contains(d);
        }

        // TODO: Remove this once performance of ASort improved
        if (s == SortEnum.ASORT) {
            return List.of(
                    DataEnum.COMPETITION_173,
                    DataEnum.COMPETITION_174
            ).contains(d);
        }

        return false;
    }
}
